/*
#ident	"@(#)smail/src:RELEASE-3_2_0_120:header.h,v 1.2 2003/12/14 22:42:41 woods Exp"
 */

/* external variables defined in header.c */
extern charplist_t *header;		/* list of header fields */

/* 
 * Local Variables:
 * c-file-style: "smail"
 * End:
 */
